//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
//
//  This code is distributed under the GNU Public License
//  which can be found at http://www.gnu.org/licenses/gpl.txt
//
//----------------------------------------------------------------------------
// File Name           : SPI_ctrl.h
// Object              : QROlleSAM7S Board Features Definition File.
//
// Creation            : NSn   20080111
//----------------------------------------------------------------------------
#ifndef SPI_CTRL_H_
#define SPI_CTRL_H_

// Include Standard files
	extern void Spi_Master ( AT91PS_SPI pSPI );
	extern void AT91F_SPI_SWReset (const AT91PS_SPI pSpi);
	extern void AT91F_Set_SPI_CSR (const AT91PS_SPI pSpi, unsigned long Csr_Value, unsigned long Csr_Number);
	extern void AT91F_SPI_Send_Data (const AT91PS_SPI pSpi, unsigned long Data_To_Send );
	extern unsigned long AT91F_SPI_Read_Data (const AT91PS_SPI pSpi);
	extern void AT91F_SPI_En_Dis (const AT91PS_SPI pSpi, unsigned char Enable);
	extern void  AT91F_SPI_Open ( AT91PS_SPI pSPI );	
	extern void AT91F_SPI_Interrupt_handler (void);
	extern int WaitUntilReady4TX ( AT91PS_SPI pSPI );
	extern int WaitUntilReady2RX ( AT91PS_SPI pSPI );
	extern void AT91_SPI_Write_MCP23S17_port ( BYTE device, BYTE subadress, BYTE portvalue );
//	extern void SPI_Write_IO_port ( BYTE device, BYTE port, BYTE * data, UINT count );
//	extern BYTE SPI_Read_IO_port ( BYTE device, BYTE port, BYTE * data, UINT count );


	extern void Var_Init (void);
	extern void Wait (unsigned long Speed);
	
	extern unsigned long volatile Number_Of_Error;
	
	// spi status flags
	extern unsigned long volatile spi_Status;		// Image of spi status register
	extern unsigned char volatile spi_Rdrf_Flag;		// spi Recieve Data Register Full  flag
	extern unsigned char volatile spi_Tdre_Flag;		// spi Transmit Data Register Empty flag
	extern unsigned char volatile spi_Modf_Flag;		// spi Mode Fault Error flag
	extern unsigned char volatile spi_Ovred_Flag;	// spi Overrun Error flag
	extern unsigned char volatile spi_Nssr_Flag;		// spi NSS rising flag
	extern unsigned long volatile spi_Txempty_Flag;	// spi Transmission Register Empty flag
	extern unsigned char volatile spi_Spiens_Flag;	// spi Enable Register flag
	extern unsigned long volatile spi_Endrx_Flag;
	extern unsigned long volatile spi_Endtx_Flag;
	extern unsigned long volatile spi_Rxbuff_Flag;
	extern unsigned long volatile spi_Txbuff_Flag;

	
	typedef	unsigned char		boolean;
        	
	#define ENABLE_SPI   				0
	#define DISABLE_SPI  				1
	#define ENABLE_DISABLE_SPI  2

	#define CSR0   						0
	#define CSR1   						1
	#define CSR2   						2
	#define CSR3					   	3

	#define MASTER 						0
	#define SLAVE 						1
	
	
	#define LASTXFER 					( 1 << 24 )
	
	

	// Addresses for SPI channels
	#define ADD_SPI_CS0						(	0x0E << 16 )
	#define ADD_VOX								(	0x0E << 16 )
	#define ADD_POT								(	0x0E << 16 )
	#define ADD_COMBO_MCP23S17		(	0x0E << 16 )
	#define ADD_MAIN_MCP23S17			(	0x0B << 16 )
	#define ADD_AT25320						(	0x07 << 16 )
	
	#define NO_ERROR_RETURNED			0

	// External interrupt sources definition
	#define 	AT91C_AIC_SRCTYPE_EXT_LOW_LEVEL  ((unsigned int) 0x0 <<  5) // (AIC) External Sources Code Label Low-level Sensitive
	#define 	AT91C_AIC_SRCTYPE_EXT_NEGATIVE_EDGE   ((unsigned int) 0x1 <<  5) // (AIC) External Sources Code Label Negative Edge triggered


#endif /*SPI_CTRL_H_*/
