//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
//
//  This code is distributed under the GNU Public License
//  which can be found at http://www.gnu.org/licenses/gpl.txt

/*----------------------------------------------------------------------------
* File Name           : eepromhandler.h
// Object              : QROlleSAM7S Board Features Definition File.
//
// Creation            : NSn   20080111
*----------------------------------------------------------------------------
*/

// Include the board file description
#include "Board.h"

#include "SPI_ctrl.h"
#include "eepromhandler.h"

BYTE *  pEEPROM; 
BYTE    test;

//+++++++++++++++++++++++++++++++++++++++++++++++++

BYTE readFromEEPROM ( WORD address_in_EEPROM, BYTE * DataAddress, WORD count  )
{
	WORD		countdown = count;//We count DOWN...
	BYTE		status;
	
	unsigned long Data_To_Send;

	WaitUntilReady4TX ( AT91C_BASE_SPI0 );//20090414
// Cfg SPI MR
		// SPI is in master mode + CS not variable + CS directly connected + 
		// no Fault mode + min DLYBCS
		AT91C_BASE_SPI0 -> SPI_MR = (AT91C_SPI_MSTR | AT91C_SPI_MODFDIS | ADD_AT25320 );
		// For device 3 ( AT25320 ) set speed to MCKL/32 and 8 bits, 
		// CS active after transfer, mode 0, 1 clk delay b4 CS up
	AT91F_Set_SPI_CSR ( AT91C_BASE_SPI0, 
										( SPI_clock_size_EEPROM << 8 ) | 
//20090315										( 9 << 8 ) | 
										AT91C_SPI_BITS_8 | 
										AT91C_SPI_NCPHA | 
										AT91C_SPI_CSAAT | 
										( 1 << 24 ), 
										CSR3 );

EEPROMnotReady:
	Data_To_Send = READ_EEPROM_STATUS_COMMAND | ADD_AT25320;
	AT91F_SPI_Send_Data ( AT91C_BASE_SPI0, Data_To_Send );
 	AT91F_SPI_Read_Data ( AT91C_BASE_SPI0 );
  while ( AT91C_BASE_SPI0->SPI_SR & AT91C_SPI_TDRE == 0 );
  //20081008while (spi_Tdre_Flag == 0);

	Data_To_Send = 0 | ( ADD_AT25320 ) | LASTXFER;
	AT91F_SPI_Send_Data ( AT91C_BASE_SPI0, Data_To_Send );
 	status = ( AT91F_SPI_Read_Data ( AT91C_BASE_SPI0 )) & 0xFF;
	while (spi_Tdre_Flag == 0);
	
	if (( status & EEPROM_ready ) != 0 )
		goto EEPROMnotReady;
		
	Data_To_Send = READ_EEPROM_COMMAND | ADD_AT25320;
	AT91F_SPI_Send_Data ( AT91C_BASE_SPI0, Data_To_Send );
	AT91F_SPI_Read_Data (AT91C_BASE_SPI0);
	while (spi_Tdre_Flag == 0);

	
	Data_To_Send = ((( address_in_EEPROM ) & 0xFF00 ) >> 8  ) | ADD_AT25320;
	AT91F_SPI_Send_Data ( AT91C_BASE_SPI0, Data_To_Send );
	AT91F_SPI_Read_Data (AT91C_BASE_SPI0);
	while (spi_Tdre_Flag == 0);
	
	Data_To_Send = (( address_in_EEPROM ) & 0xFF ) | ADD_AT25320;
	AT91F_SPI_Send_Data ( AT91C_BASE_SPI0, Data_To_Send );
	AT91F_SPI_Read_Data (AT91C_BASE_SPI0);
	while (spi_Tdre_Flag == 0);
	
	for ( ; countdown > 0; countdown--)
	{
		if ( countdown == 1 )
			Data_To_Send = 0 | ADD_AT25320 | LASTXFER;
		else
			Data_To_Send = 0 | ADD_AT25320;
		AT91F_SPI_Send_Data ( AT91C_BASE_SPI0, Data_To_Send );
		*DataAddress++ = (( AT91F_SPI_Read_Data ( AT91C_BASE_SPI0 )) & 0xFF );
	while (spi_Tdre_Flag == 0);
	};

	return *--DataAddress;
};

//+++++++++++++++++++++++++++++++++++++++++++++++++

void  send2EEPROM ( WORD address_in_EEPROM, BYTE * DataAddress, WORD count )
{
	BYTE						status;
//	BYTE						countdown = count & 0x3F; //NOT more than 32 bytes
	BYTE						countdown = count; //WHY NOT more than 32 bytes
	unsigned long 	Data_To_Send;
	WORD 						address = address_in_EEPROM;

	WaitUntilReady4TX ( AT91C_BASE_SPI0 );//20090414
		// Cfg SPI MR
		// SPI is in master mode + CS not variable + CS directly connected + 
		// no Fault mode + min DLYBCS
		AT91C_BASE_SPI0 -> SPI_MR = (AT91C_SPI_MSTR | AT91C_SPI_MODFDIS | ADD_AT25320 );
		// For device 3 ( EEPROM ) set speed to MCKL/4 and 8 bits, 
		// CS active after transfer, mode 0, 1 clk delay b4 CS up
	AT91F_Set_SPI_CSR ( AT91C_BASE_SPI0, 
//20090315										( 9 << 8 ) | 
										( SPI_clock_size_EEPROM << 8 ) | 
										AT91C_SPI_BITS_8 | 
										AT91C_SPI_NCPHA | 
										AT91C_SPI_CSAAT | 
										( 1 << 24 ), 
										CSR3 );
	
OneMoreTime:
	Data_To_Send = READ_EEPROM_STATUS_COMMAND | ( ADD_AT25320 );
	AT91F_SPI_Send_Data ( AT91C_BASE_SPI0, Data_To_Send );
	status = ( AT91F_SPI_Read_Data (AT91C_BASE_SPI0) & 0xFF );
	while (spi_Tdre_Flag == 0);
	
	Data_To_Send = 0 | ( ADD_AT25320 ) | LASTXFER;
	AT91F_SPI_Send_Data ( AT91C_BASE_SPI0, Data_To_Send );
	status = ( AT91F_SPI_Read_Data (AT91C_BASE_SPI0) & 0xFF );
	while (spi_Tdre_Flag == 0);
	
	if (( status & EEPROM_ready ) != 0 )
		goto OneMoreTime;
		
	Data_To_Send = WRITE_ENABLE_COMMAND | ( ADD_AT25320 ) | LASTXFER;
	AT91F_SPI_Send_Data ( AT91C_BASE_SPI0, Data_To_Send );
	AT91F_SPI_Read_Data (AT91C_BASE_SPI0);
	while (spi_Tdre_Flag == 0);
	
	Data_To_Send = READ_EEPROM_STATUS_COMMAND | ( ADD_AT25320 );
	AT91F_SPI_Send_Data ( AT91C_BASE_SPI0, Data_To_Send );
	status = ( AT91F_SPI_Read_Data (AT91C_BASE_SPI0) & 0xFF );
	while (spi_Tdre_Flag == 0);

	Data_To_Send = 0 | ( ADD_AT25320 ) | LASTXFER;
	AT91F_SPI_Send_Data ( AT91C_BASE_SPI0, Data_To_Send );
	status = ( AT91F_SPI_Read_Data (AT91C_BASE_SPI0) & 0xFF );
	while (spi_Tdre_Flag == 0);

	if (( status & EEPROM_ready ) != 0 )
		goto OneMoreTime;
		
	Data_To_Send = WRITE_EEPROM_COMMAND | ( ADD_AT25320 );
	AT91F_SPI_Send_Data ( AT91C_BASE_SPI0, Data_To_Send );
	AT91F_SPI_Read_Data (AT91C_BASE_SPI0);
	while (spi_Tdre_Flag == 0);
	
	Data_To_Send = ((( address ) & 0xFF00 ) >> 8  ) | ( ADD_AT25320 );
	AT91F_SPI_Send_Data ( AT91C_BASE_SPI0, Data_To_Send );
	AT91F_SPI_Read_Data (AT91C_BASE_SPI0);
	while (spi_Tdre_Flag == 0);
	
	Data_To_Send = (( address ) & 0xFF ) | ( ADD_AT25320 );
	AT91F_SPI_Send_Data ( AT91C_BASE_SPI0, Data_To_Send );
	AT91F_SPI_Read_Data (AT91C_BASE_SPI0);
	while (spi_Tdre_Flag == 0);
	

	for ( ; countdown > 0; countdown--)
	{
		address++;
		if (( address & 0x1F ) == 0 ) //Coming to next subpage
			Data_To_Send = *DataAddress++  | ADD_AT25320 | LASTXFER;
		else if ( countdown == 1 )
			Data_To_Send = *DataAddress++  | ADD_AT25320 | LASTXFER;
		else
			Data_To_Send = *DataAddress++ | ADD_AT25320;
		AT91F_SPI_Send_Data ( AT91C_BASE_SPI0, Data_To_Send );
		AT91F_SPI_Read_Data (AT91C_BASE_SPI0);
		while (spi_Tdre_Flag == 0);
		if (( address & 0x1F ) == 0 ) //Coming to next subpage
			goto OneMoreTime;
	};

};

//+++++++++++++++++++++++++++++++++++++++++++++++++
