//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
//
//  This code is distributed under the GNU Public License
//  which can be found at http://www.gnu.org/licenses/gpl.txt
//
//----------------------------------------------------------------------------
// File Name           : main.c
// Object              : QROlleSAM7X Board Features Definition File.
//
// Creation            : NSn   20081111
// 
//----------------------------------------------------------------------------
 
//  *******************************************************
//                Header Files
//  *******************************************************
#include <string.h> 
#include <inttypes.h>
#include "Board.h"
#include "display.h"
#include "eepromhandler.h"
#include "SPI_ctrl.h"
#include "HandleDDS.h"
#include "menu.h"
#include "a2dmodule.h"
#include "commands.h"
#include "cdc_enumerate.h"
#include "Smeter.h"
#include "serialARM.h"

const char cCompileTime [] =   __TIME__;
const char cCompileDate [] =   __DATE__;
const char cVersion []  = {"SW   033"};

// Version [] = "SW 7S033  ";  Non-blinking 'N'
// Version [] = "SW 7S032  ";  SPI: now divide-by-11
// Version [] = "SW 7S031  ";  SPI: now divide-by-8, serial com improved
// Version [] = "SW 7S030  ";  Further SPI changes, now to divide-by-63
// Version [] = "SW 7S029  ";  Further SPI changes, now back to divide-by-8
// Version [] = "SW 7S028  ";  Further SPI changes   divide-by-255
// Version [] = "SW 7S027  ";  SPI changes
// Version [] = "SW 7S026  ";  Serial Buffers from 256, now 4096 bytes... Now FT-2000 compatible
// Version [] = "SW 7S025  ";  CAT commands better...
// Version [] = "SW 7S024  ";  A bit better...
// Version [] = "SW 7S023  ";  RIT button moved from subfunction to own button
// Version [] = "SW 7S022  ";  Buzzer active
// Version [] = "SW 7S021b ";  smaller bugs corrected: Voxdelay, powersetting etc
// Version [] = "SW 7S021a ";  small bugs corrected
// Version [] = "SW 7S021 ";  >20 new commands implemented
// Version [] = "SW 7S020 ";  Serial port baudrate change
// Version [] = "SW 7S019 ";  Serial port changed to USART0 instead of Debug
// Version [] = "SW 7S018 ";  improved menu system w "Brje" methode. Same as 7S017f
// Version [] = "SW 7X017a ";  "is OFF" borta?
// Version [] = "SW 7X017 ";  Hold time problem solved
// Version [] = "SW 7X016 ";  Hold time now from end of CW pulse, hold time min set in Board.h
// Version [] = "SW 7X015 ";  Menu handling with encoder...
// Version [] = "SW 7X014 ";  First for 7000X with new hardware...
// Version [] = "SW xx013 ";  Correct bargraph...
// Version [] = "SW xx012 ";  Correct output to diode...
// Version [] = "SW xx011 ";  Correct table...
// Version [] = "SW xx010 ";  New Powerregulation table

// Version [] = "SW xx008 ";  For new hardware
// Version [] = "SW xx007 ";  Improved PTT
// Version [] = "SW 1x000 ";  First AT91SAM7X version with old hw
// Version [] = "SW 7x006 ";  First AT91SAM7X version
// Version [] = "SW xx006 ";  RIT-button now on PreAmpButton
// Version [] = "SW xx005 ";  powersetting now shown on display!!!!!!!!
// Version [] = "SW xx004 ";  always powersetting; No fq-mode change at start
// Version [] = "SW xx003 ";  displaying error corrected
// Version [] = "SW xx002 ";  small changes!!!!!
// Version [] = "SW xx001 ";  small changes!!!!!
// Version [] = "SW xx000 ";  generic sw: Chosen hardware, selected in Board.h
//const char cVersion []  = "SW v.M00 ";
//const char cVersion [] =   "Safety 14";

// Version [] = "SW v.M00 ";  TWI (I2C) removed
// Version [] = "SW v.U22 ";  Faster SPI-com (>= 16 times)
// Version [] = "SW v.U19 ";  PTT inhibited when CW training
// Version [] = "SW v.U18 ";  with tuning limitations
// Version [] = "SW v.U17 ";  with update of FQ at start in CW mode
// Version [] = "SW v.U16 ";  with adjustable hold time
// Version [] = "SW v.U15 ";  select bug right or left handed
// Version [] = "SW v.U14 ";  small bug repairs
// Version [] = "SW v.U13 ";  interrupt setup change for bug use buggering me
// Version [] = "SW v.U12 ";  with straight keying via bug connector input
// Version [] = "SW v.U11 ";  with Iambic keying
//

//  *******************************************************
//                Function Prototypes
//  *******************************************************
void Timer0IrqHandler(void);
void Timer1IrqHandler(void);
void Timer2IrqHandler(void);
void FiqHandler(void);

//  *******************************************************
//                External References
//  *******************************************************
extern	void LowLevelInit(void);
extern	void TimerSetup(void);
extern	unsigned enableIRQ(void);
extern	unsigned enableFIQ(void);
extern  void USART0Setup(void);
extern void AT91F_US_Put( char *buffer); // \arg pointer to a string ending by \0
extern void Trace_Toggel_LED( unsigned int led) ;
extern void AT91F_LowLevelInit(  void ) ;

//  *******************************************************
//               Global Variables
//  *******************************************************

	struct _AT91S_CDC 	pCDC;
	// pointer to PIO data structure
	volatile AT91PS_PIO	pPIO = AT91C_BASE_PIOA;
	volatile AT91PS_SPI 	pSpi= AT91C_BASE_SPI0;
	
	
	volatile WORD  				wCheck4eeprom = 0;
	volatile BYTE  				idx = 0;
	volatile WORD  				ShowVolume = 0;
	volatile WORD  				ShowPower = 0;
	volatile BYTE  				dummy = 0;
	volatile BYTE         				Repeat_Timing = 0;
	volatile UINT					timeSinceLastSave = 0;
	volatile BYTE					AD9834dataDirty = 0;
	volatile BYTE					POT1dataDirty = 0;
 	 volatile BYTE        				 PowerdataDirty = 0;
	volatile BYTE					ADCchanneldataDirty = 0;
	
	volatile UINT					EncoderDirection = 0;
	volatile UINT					TimeInMainLoop = 0;
	struct Parameters 			param;
	
	
	volatile WORD					wBugSpeed;
	volatile WORD					wDisplayBackLiteTime;
	volatile BYTE					timing0 = 0;
	volatile BYTE					displayTiming = 0;
	volatile BYTE					ModeChanged = 0; 
	volatile BYTE					mainmode = MENUMODE;

	volatile UINT		 			AD9834RefFrequency; 
	volatile UINT		 			AD9834RIT_Afrequency; 
	volatile UINT		 			AD9834RIT_Bfrequency; 
	volatile BYTE					OutputdataDirty;
	volatile BYTE					ComboIOportAIn;
	volatile BYTE					ComboIOportBIn;
	volatile BYTE					ButtonActionFlt;
	volatile BYTE					ButtonActionPre;
	volatile BYTE					ButtonActionMd;
	volatile BYTE					ButtonActionCom;
	volatile BYTE					ButtonActionABSplit;
	volatile BYTE					ButtonActionBdw;
	volatile BYTE					ButtonActionstep;
	volatile BYTE					ButtonActionBup;
	volatile BYTE					DisplayMode = DisplayOn;
	volatile BYTE					BytesReceivedUSB;
	volatile BYTE					BytesInBufferUSB;
	volatile BYTE					InBufferUSB [ 64 ];
	unsigned int						FiqCount = 0;		// global uninitialized variable		
	volatile UINT 					length;
  volatile BYTE         BugConnection;
  volatile BYTE         BUG_Conn0_ACTIVE;
  volatile BYTE         BUG_Conn1_ACTIVE;
	
	// pointer to PMC data structure
	volatile AT91PS_PMC	pPMC = AT91C_BASE_PMC;
	
	// pointer to AIC data structure  
	volatile AT91PS_AIC	pAIC = AT91C_BASE_AIC;
	
//  *******************************************************
	
void WelcomeMsg (void )
{
	BYTE  potvol;
	
	memset ( (char*)&DisplayBuffer [ 0 ], ' ', CHAR_ON_DISPLAY );
	TimeInMainLoop = NUMBERof5msDELAYS * 100 ;
	memcpy ( (char*)&DisplayBuffer [ 0 ], &cVersion [0], strlen ( cVersion ) );
  DisplayBuffer [ 3 ] = GENERATION;
  DisplayBuffer [ 4 ] = COMBOGENERATION;
  memcpy ( (char*)&DisplayBuffer [ CHAR_ON_ONE_LINE ], 
          &cCompileTime [0], 
          strlen ( cCompileTime ));
  memcpy ( (char*)&DisplayBuffer [ CHAR_ON_ONE_LINE + strlen ( cCompileTime ) + 2 ], 
          &cCompileDate [0], 
          strlen ( cCompileDate ) - 4 );
	memcpy ( (char*)&DisplayBuffer [ CHAR_ON_ONE_LINE +
	                                 strlen ( cCompileTime ) + 2 +
	                                 strlen ( cCompileDate ) - 3 ], 
					&cCompileDate [strlen ( cCompileDate ) - 2 ], 2 );
	DisplayBuffer [ CHAR_ON_ONE_LINE + 
                  strlen ( cCompileTime ) + 2 +
                  strlen ( cCompileDate ) - 4 ] = '-';
	Write2display();
	SetCursorPosition ( CHAR_ON_DISPLAY );
	potvol = Pot1Volume;
	Pot1Volume = 0;
	SetPot1Volume();
	while ( TimeInMainLoop );
	Pot1Volume = potvol;
	SetPot1Volume();
WelcomeMsgOut:
	mainmode = RXmode;
	SetPot1Volume();
};

//*----------------------------------------------------------------------------
//* \fn    AT91F_USB_Open
//* \brief This function Open the USB device
//*----------------------------------------------------------------------------
void AT91F_USB_Open(void)
{
    // Set the PLL USB Divider
    AT91C_BASE_CKGR->CKGR_PLLR |= AT91C_CKGR_USBDIV_1 ;

    // Specific Chip USB Initialisation
    // Enables the 48MHz USB clock UDPCK and System Peripheral USB Clock
    AT91C_BASE_PMC->PMC_SCER = AT91C_PMC_UDP;
    AT91C_BASE_PMC->PMC_PCER = (1 << AT91C_ID_UDP);

    // Enable UDP PullUp (USB_DP_PUP) : enable & Clear of the corresponding PIO
    // Set in PIO mode and Configure in Output
    AT91F_PIO_CfgOutput ( AT91C_BASE_PIOA, aUSBon ); //Done in init!
    // Clear for set the Pul up resistor
    AT91F_PIO_ClearOutput ( AT91C_BASE_PIOA, aUSBon );

    // CDC Open by structure initialization
    AT91F_CDC_Open ( &pCDC, AT91C_BASE_UDP );
}

//  *******************************************************
//                     Initialization
//  ******************************************************/

void InitController ( void )
{

//	BYTE			idx, dummy;
	
	 // Enable User Reset and set its minimal assertion to 960 us
	 AT91C_BASE_RSTC->RSTC_RMR = AT91C_RSTC_URSTEN | (0x4<<8) | (unsigned int)(0xA5<<24);
	
	// Initialize the Atmel AT91SAM7S256 (watchdog, PLL clock, default interrupts, etc.)
	// ---------------------------------------------------------------------------------
	LowLevelInit();
//	LowLevelInit();
	
	
	// Turn on the peripheral clock for Timers
	// ---------------------------------------
	
	// enable Timer0 peripheral clock		
	pPMC->PMC_PCER = (1<<AT91C_ID_TC0);	
	// enable Timer1 peripheral clock		
	pPMC->PMC_PCER = (1<<AT91C_ID_TC1);	
	// enable Timer2 peripheral clock		
	pPMC->PMC_PCER = (1<<AT91C_ID_TC2);	
	
	// Set up the PIO ports
	// --------------------			
   // First, enable the clock of the PIOA and PIOB
  AT91F_PMC_EnablePeriphClock ( AT91C_BASE_PMC, 
															( 1 << AT91C_ID_PIOA ) |
															( 1 << AT91C_ID_PIOB )) ;
	

	AT91F_PIO_ClearOutput ( AT91C_BASE_PIOA,aIOKE );
	AT91F_PIO_SetOutput ( AT91C_BASE_PIOA, aUSBon );  //USBon active low!!!!!
	// PIO Enable Registers - allow PIO to control pins - sets pins to outputs	
	AT91F_PIO_CfgOutput	( AT91C_BASE_PIOA, 
												aDualLEDred |
												aDualLEDgreen |
                        aDRd |
                        aDReg |
												aHC139A |
												aHC139B |
												aIOKE |
												aUSBon |
												aSummer );
AT91F_PIO_SetOutput ( AT91C_BASE_PIOA, aDualLEDred );
AT91F_PIO_ClearOutput ( AT91C_BASE_PIOA, aDualLEDgreen );
  
	AT91F_PIO_CfgOutput	( AT91C_BASE_PIOB, 
                       bCharacterBitMask |
                       bDEna |
											 bEnaIOKE |
	                     bDISP_BackLite  );

	// PIO Enable Registers - allow PIO to control pins - sets pins to inputs	
	AT91F_PIO_CfgInput	( AT91C_BASE_PIOA, 
												aSW_MASK | 
												aUSBdetect  | 
                        aENCODER_MASK | 
                        aENCODERSW );
	AT91F_PIO_CfgInput	( AT91C_BASE_PIOB, 
												bSW_MASK | 
                        bInterrupt0 | 
                        bInterrupt1 |
												bPlusTx );
	AT91F_PIO_CfgInputFilter ( AT91C_BASE_PIOA, 
														aSW_MASK | 
														aUSBdetect |
                            aENCODER_MASK | 
                            aENCODERSW );
	AT91F_PIO_CfgInputFilter ( AT91C_BASE_PIOB, 
                              bInterrupt0 | 
                              bInterrupt1); //Enable filter 
	
  AT91C_BASE_PIOA->PIO_PPUDR = aUSBdetect; //Disable pullup on USBdetect
  AT91C_BASE_PIOB->PIO_PPUDR = bPlusTx; //Disable pullup on +TX 
	AT91F_PIO_CfgPullup ( AT91C_BASE_PIOA, 
												aSW_MASK ); //Enable pullup 
	AT91F_PIO_CfgPullup ( AT91C_BASE_PIOB, 
                        bInterrupt0 | 
                        bInterrupt1); //Enable pullup 

  /* Disable the interrupt on the interrupt controller */
AT91C_BASE_AIC->AIC_IDCR = ( 1 << AT91C_ID_PIOA ) | ( 1 << AT91C_ID_PIOB );

  /* Save the interrupt handler routine pointer and the interrupt priority */
//20080929  AT91C_BASE_AIC->AIC_SVR [ AT91C_ID_PIOA ] = (UINT) bug_handler;

//		if ( SEVENTH_BUTTON ) bootloader();




/* Store the Source Mode Register */
//  AT91C_BASE_AIC->AIC_SMR [ AT91C_ID_PIOA ] = AT91C_AIC_SRCTYPE_INT_HIGH_LEVEL | AT91C_AIC_PRIOR_HIGHEST;
  AT91C_BASE_AIC->AIC_SMR [ AT91C_ID_PIOA ] = AT91C_AIC_SRCTYPE_HIGH_LEVEL | AT91C_AIC_PRIOR_LOWEST;
  
  /* Clear the interrupt on the interrupt controller */
  AT91C_BASE_AIC->AIC_ICCR = ( 1 << AT91C_ID_PIOA )|( 1 << AT91C_ID_PIOB );

	// PIO Set Output Data Register - turns off the LEDs						
	AT91F_PIO_SetOutput ( AT91C_BASE_PIOA, aDualLEDred );
	AT91F_PIO_ClearOutput ( AT91C_BASE_PIOA, aDualLEDgreen );
	AT91F_PIO_ClearOutput ( AT91C_BASE_PIOA, aSummer );
	
	SetupAT91SAM_ADC ();
	
	//  Cfp SPI PIO, PMC, ...
	
	AT91F_PIOA_CfgPMC ();  //To get the input function to run!!!!!!!!!!!!!!!!!!!!!!!!!!
	
	// Configure SPI PIOs
	AT91F_SPI0_CfgPIO ();  

	// Configure PMC by enabling SPI clock
	AT91F_SPI0_CfgPMC ();

 	// Configure TWI aka I2C
	AT91F_TWI_CfgPMC (); //Enable peripheral clock for TWI
	AT91F_TWI_CfgPIO (); // Enable PIO pins for TWI
	AT91F_PIO_CfgOpendrain ( AT91C_BASE_PIOA, (unsigned int) AT91C_PA10_TWD );
	AT91F_TWI_Configure ( AT91C_BASE_TWI ); // Set TWI to master mode
	// Set TWI to 400kHz
	AT91C_BASE_TWI->TWI_CWGR = (( 3 << 16 ) | ( 0x07 << 8 ) | 0x07 ); // CKDIV | CHDIV | CKDIV
	// Set up the Advanced Interrupt Controller AIC for Timer 0,1,2
	// --------------------------------------------------------
	
	// Disable timer 0,1,2 interrupt in AIC Interrupt Disable Command Register		
	pAIC->AIC_IDCR = (1<<AT91C_ID_TC0) | (1<<AT91C_ID_TC1) | (1<<AT91C_ID_TC2);												
	
	// Set the TC0,1,2 IRQ handler address in AIC Source Vector Register[12]
	pAIC->AIC_SVR [ AT91C_ID_TC0 ] = (UINT) Timer0IrqHandler;				
	pAIC->AIC_SVR [ AT91C_ID_TC1 ] = (UINT) Timer1IrqHandler;				
	pAIC->AIC_SVR [ AT91C_ID_TC2 ] = (UINT) Timer2IrqHandler;				
	
	// Set the interrupt source type and priority (0-7) in AIC Source Mode Register[12]
	pAIC->AIC_SMR [ AT91C_ID_TC0 ] = ( AT91C_AIC_SRCTYPE_INT_HIGH_LEVEL | 0x4 );	
	pAIC->AIC_SMR [ AT91C_ID_TC1 ] = ( AT91C_AIC_SRCTYPE_INT_HIGH_LEVEL | 0x4 );	
	pAIC->AIC_SMR [ AT91C_ID_TC2 ] = ( AT91C_AIC_SRCTYPE_INT_HIGH_LEVEL | 0x4 );	
	
	// Clear the TC0,1,2 interrupt in AIC Interrupt Clear Command Register
	pAIC->AIC_ICCR = ( 1<<AT91C_ID_TC0 ) | ( 1<<AT91C_ID_TC1 ) | ( 1<<AT91C_ID_TC2 ) ; 										
	
	// Remove disable timer 0,1,2 interrupt in AIC Interrupt Disable Command Register			
	pAIC->AIC_IDCR = ( 0 << AT91C_ID_TC0 ) | ( 0 << AT91C_ID_TC1 ) | ( 0 << AT91C_ID_TC2 );											
	
	// Enable the TC0,1,2 interrupt in AIC Interrupt Enable Command Register
	pAIC->AIC_IECR = ( 1<<AT91C_ID_TC0 ) | ( 1<<AT91C_ID_TC1 ) | ( 1<<AT91C_ID_TC2 ) ; 										
	
	
	// Set up the Advanced Interrupt Controller AIC for SPI
	// ----------------------------------------------------
	
		AT91F_AIC_ConfigureIt ( AT91C_BASE_AIC, 
															AT91C_ID_SPI0, 
														AT91C_AIC_PRIOR_HIGHEST,
															AT91C_AIC_SRCTYPE_INT_POSITIVE_EDGE, 
														AT91F_SPI_Interrupt_handler);
		  AT91F_AIC_EnableIt (AT91C_BASE_AIC, AT91C_ID_SPI0 );	

	// Set up the Advanced Interrupt Controller AIC for FIQ 
	// ---------------------------------------------------------------------
	
    // Disable FIQ interrupt in AIC Interrupt Disable Command Register	
	pAIC->AIC_IDCR = (1<<AT91C_ID_FIQ);													
	
	// Set the interrupt source type in AIC Source Mode Register[0]
	pAIC->AIC_SMR[AT91C_ID_FIQ] = (AT91C_AIC_SRCTYPE_INT_POSITIVE_EDGE);		
	
	// Clear the FIQ interrupt in AIC Interrupt Clear Command Register
	pAIC->AIC_ICCR = (1<<AT91C_ID_FIQ); 										
	
	// Remove disable FIQ interrupt in AIC Interrupt Disable Command Register		
//	pAIC->AIC_IDCR = (0<<AT91C_ID_FIQ);												
	
	// Enable the FIQ interrupt in AIC Interrupt Enable Command Register
//	pAIC->AIC_IECR = (1<<AT91C_ID_FIQ); 		
	
  /* Enable PIO interrupt in the interrupt controller */
  AT91C_BASE_AIC->AIC_IECR = (1 << AT91C_ID_PIOA);

	// USB setup
	
  AT91C_BASE_CKGR->CKGR_PLLR |= AT91C_CKGR_USBDIV_1 ;

   // Specific Chip USB Initialisation
   // Enables the 48MHz USB clock UDPCK and System Peripheral USB Clock
   AT91C_BASE_PMC->PMC_SCER = AT91C_PMC_UDP;
   AT91C_BASE_PMC->PMC_PCER = (1 << AT91C_ID_UDP);

   // Enable UDP PullUp (USB_DP_PUP) : enable & Clear of the corresponding PIO
   // Set in PIO mode and Configure in Output
   AT91F_PIO_CfgOutput ( AT91C_BASE_PIOA, aUSBon ); //Done in init!
   // Clear for set the Pul up resistor
   AT91F_PIO_ClearOutput ( AT91C_BASE_PIOA, aUSBon );

		// Set up SPI controller MUST BE SET UP AFTER AIC!!!!!!!!!
	//
	Spi_Master ( pSpi);
	
	// Keep the below settings here so fq and mode are not stepping at start
	// They must be initialized BEFORE timer interrupt is started!!
	EncoderDirection = CLOCKWISE;
	EncoderCW_count 	= MAXENCODERCOUNT;  // So no change via encoder value!!!
	EncoderCCW_count 	= MAXENCODERCOUNT;

// Setup timers to generate a 50 msec periodic interrupt and other
	// -----------------------------------------------------
	
	TimerSetup();
	  
// enable interrupts
	// -----------------
	
	enableIRQ();
	
//	enableFIQ();

	
	// PIO Set Output Data Register - the LEDs						
	AT91F_PIO_ClearOutput ( AT91C_BASE_PIOA, aDualLEDred );
	AT91F_PIO_SetOutput ( AT91C_BASE_PIOA, aDualLEDgreen );
	IncreaseFQ = 0;
	ChangeSizePosition = 4;  // 100 Hz change
	ChangeDirection = 0;
	handleMenuItem = 0;
	CursorJump = 0;
	ControlChanged = 0;
	ValueChanged = 0;
	subfunction = 0;
	subfunctiontiming = 0;
	
//	readFromEEPROM ( AddSavedFQvalues, (BYTE*)&RAM.f1Hz, sizeof ( struct FQvalues )  );
	
	readFromEEPROM ( LastBand, 							(BYTE*)&Band, 1 );
	readFromEEPROM ( AddBugSpeed, 					(BYTE*)&wBugSpeed, 1 );
	readFromEEPROM ( AddPot1Volume, 				(BYTE*)&Pot1Volume, 1 );
	readFromEEPROM ( AddwUsedDelay, 				(BYTE*)&wUsedDelay, 2);
	readFromEEPROM ( AddwCheck4eeprom, 			(BYTE*)&wCheck4eeprom, 2 );
	readFromEEPROM ( AddwSmeterValues, 			(BYTE*)&ramSmeterValues, 24 );
	readFromEEPROM ( AddSMeterTyp, 					(BYTE*)&SMeterMode, 1 );
	readFromEEPROM ( AddAD9834RefFrequency, (BYTE*)&AD9834RefFrequency, 4 );
	readFromEEPROM ( AddDVMReference, 			(BYTE*)&DVMReference, 2 );
	readFromEEPROM ( AddIambicMode, 				(BYTE*)&bIambicMode, 1 );
	readFromEEPROM ( AddBugConnectionMode, 	(BYTE*)&BugConnection, 1 );
	readFromEEPROM ( AddbBaudrate, 					(BYTE*)&bBaudrate, 1 );
	readFromEEPROM ( AddwButtonSoundLen, 		(BYTE*)&wButtonSoundLen, 2 );
	readFromEEPROM ( AddbSMeterDisplay, 		(BYTE*)&bSMeterDisplay, 1 );

	if ( wCheck4eeprom != 0xA5A5 ) // i.e. EEPROM not initialized
	{
		param.AD9834Afrequency = 1840000; 
		param.AD9834Bfrequency = 1840000 + 500; 
		param.AD9834Splitfrequency = 1840000; 
		param.IOportA = ( IOBW | IO80 );
		param.IOportB = ( IOSB );
		param.ADCchannel = 0; 
		param.FQstepLen = SMALL_FQ_STEPS;
		param.bPowerSetting = 128;
		param.Modes = NonRITmode | AFQmode | WIDEmode | LSBmode;
		send2EEPROM ( SavedParametersB160, (BYTE*)&param, sizeof ( struct Parameters ) );
		param.AD9834Afrequency = 3742000; 
		param.AD9834Bfrequency = 3742000 + 5000; 
		param.AD9834Splitfrequency = 3742000; 
		send2EEPROM ( SavedParametersB80, (BYTE*)&param, sizeof ( struct Parameters ));
		param.AD9834Afrequency = 7075000; 
		param.AD9834Bfrequency = 7075000 + 5000; 
		param.AD9834Splitfrequency = 7075000; 
		send2EEPROM ( SavedParametersB40, (BYTE*)&param, sizeof ( struct Parameters ));
		param.AD9834Afrequency = 10140000; 
		param.AD9834Bfrequency = 10140000 + 500; 
		param.AD9834Splitfrequency = 10140000; 
		send2EEPROM ( SavedParametersB30, (BYTE*)&param, sizeof ( struct Parameters ));
		param.AD9834Afrequency = 14200000; 
		param.AD9834Bfrequency = 14200000 + 5000; 
		param.AD9834Splitfrequency = 14200000; 
		param.Modes = NonRITmode | AFQmode | WIDEmode | USBmode;
		send2EEPROM ( SavedParametersB20, (BYTE*)&param, sizeof ( struct Parameters ));
		param.AD9834Afrequency = 18120000; 
		param.AD9834Bfrequency = 18120000 + 500; 
		param.AD9834Splitfrequency = 18120000; 
		send2EEPROM ( SavedParametersB17, (BYTE*)&param, sizeof ( struct Parameters ));
		param.AD9834Afrequency = 14200000; 
		param.AD9834Bfrequency = 14200000 + 5000; 
		param.AD9834Splitfrequency = 14200000; 
		send2EEPROM ( SavedParametersTrans, (BYTE*)&param, sizeof ( struct Parameters ));
		Band = B80; 
		send2EEPROM ( LastBand, (BYTE*)&Band, 1 );
		Pot1Volume = 0; 
		send2EEPROM ( AddPot1Volume, (BYTE*)&Pot1Volume, 1 );
		wBugSpeed = MINBUGSPEED; 
		send2EEPROM ( AddBugSpeed, (BYTE*)&wBugSpeed, 1 );
		wUsedDelay = SHORTDELAY;
		send2EEPROM ( AddwUsedDelay, (BYTE*)&wUsedDelay, 2 );
		memcpy ( (char*)ramSmeterValues, &cSmeterValues[0], 24 );
		send2EEPROM ( AddwSmeterValues, (BYTE*)&ramSmeterValues, 24 );
		SMeterMode = GRAPHIC;
		send2EEPROM ( AddSMeterTyp, (BYTE*)&SMeterMode, 1 );
		AD9834RefFrequency = 100000000;
		send2EEPROM ( AddAD9834RefFrequency, (BYTE*)&AD9834RefFrequency, 4 );
		bIambicMode = NoIambicMode;
		send2EEPROM ( AddIambicMode, 	(BYTE*)&bIambicMode, 1 );
		BugConnection = 0;
		send2EEPROM ( AddBugConnectionMode, (BYTE*)&BugConnection, 1 );
		bBaudrate = 0;
		send2EEPROM ( AddbBaudrate, (BYTE*)&bBaudrate, 1 );
		wButtonSoundLen = 0;
		send2EEPROM ( AddwButtonSoundLen, (BYTE*)&wButtonSoundLen, 2 );
		bSMeterDisplay = 0;
		send2EEPROM ( AddbSMeterDisplay, (BYTE*)&wButtonSoundLen, 2 );

		wCheck4eeprom = 0xA5A5;
		send2EEPROM ( AddwCheck4eeprom, (BYTE*)&wCheck4eeprom, 2 );
	};

	//Set pins in Main and Combo IO as outputs
#ifdef QROLLE_COMBO_SPI
  dummy = ComboIO_VALUE_OUTPUT_CONFIG_A;
	Write_Combo_IO ( A_CONFIG, (BYTE*)&dummy, 1 );
  dummy = ComboIO_VALUE_OUTPUT_CONFIG_B;
  Write_Combo_IO ( B_CONFIG, (BYTE*)&dummy, 1 );
#endif
#ifdef QROLLE_COMBO_TWI
  dummy = ComboIO_VALUE_OUTPUT_CONFIG_A;
	Write_Combo_IO ( A_CONFIG, (BYTE*)&dummy, 1 );
  dummy = ComboIO_VALUE_OUTPUT_CONFIG_B;
  Write_Combo_IO ( B_CONFIG, (BYTE*)&dummy, 1 );
#endif

	OutputdataDirty = 1;
	OpenKey;
	AT91F_PIO_ClearOutput ( AT91C_BASE_PIOB, bEnaIOKE );  //Enable key output

	SetBandData ( Band );
	param.Modes &= ~RITmode;
	CalculateDDSfractions ( AD9834RefFrequency );
	SetTimer4Bug ( wBugSpeed );

	// Setup DDS_chip
	// ---------------
	CalculateFrequencyDivisor ( param.AD9834Afrequency );
	init_DDS_chip ( pSpi); // Using interrupts....
	
	SwitchONbacklite;	
	DisplayTiming = 10; //Wait > 30 ms after power on for display	
//	AT91F_PIO_SetOutput ( AT91C_BASE_PIOA, DualLEDgreen );
	while ( DisplayTiming );

	SendCmdToDisplay ( FUNCTIONSET, NORMALEXETIME );//39us Using interrupts....
	SendCmdToDisplay ( DISPLAYSET, NORMALEXETIME ); //39us
	SendCmdToDisplay ( DISPLAYCLEAR, EXTENDEDEXETIME ); //1,53ms
	SendCmdToDisplay ( DISPLAYINCRMODE, EXTENDEDEXETIME );//1,53ms
	MakeSmeterCharacters();
	lastDisplayPosition = 0;
	LastValidInport = 0;
//20080929	if ( bIambicMode == StraightKey )
	{
			/* Enable button interrupts generation through the PIO controller */
//2008092920080929		AT91C_BASE_PIOA->PIO_IER = ( Interrupt0 | IOInt );	
		/* Enable PIO interrupt in the interrupt controller */
//20080929		AT91C_BASE_AIC->AIC_IECR = (1 << AT91C_ID_PIOA);
	};
	USART0Setup();
	WelcomeMsg ();
	for ( idx = 0; idx <  CHAR_ON_DISPLAY; idx++ )
		DisplayBuffer[ idx ] = ' ';
	for ( idx = 0; idx <  CHAR_ON_DISPLAY; idx++ )
		LastDisplayBuffer[ idx ] = 0x0;
};


//  *******************************************************
//                     MAIN
//  ******************************************************

int	main (void) 
{
	
	InitController ();
	InitMenu();
	
	for ( ; ; ) // Better than while
  {  
    while ( TimeInMainLoop )
		{
		 	if (( AT91F_PIO_IsInputSet ( AT91C_BASE_PIOA, aUSBdetect ) == aUSBdetect )) 
			{
		    // Init USB device
			  if (!AT91F_CDC_CheckCurrentConfiguration ( &pCDC ))
			  {
          AT91F_CDC_Open ( &pCDC, AT91C_BASE_UDP );
          while (!pCDC.IsConfigured(&pCDC))
          {
            if ( !( AT91F_PIO_IsInputSet ( AT91C_BASE_PIOA, aUSBdetect ) == aUSBdetect ))
            {
              AT91F_CDC_Close ( &pCDC );
              goto SkipConfiguration;             
            };
          };    
			  };
			}
			else
      {
        if ( AT91F_CDC_CheckCurrentConfiguration ( &pCDC ))
          AT91F_CDC_Close ( &pCDC );
      };

SkipConfiguration:
			if (( param.Modes &  LSBmode ) != CWmode )
			{
				if ( bSet2transmit )
				{
					SetIOPTT;
				}
				else
				{
					ResetIOPTT;
				};
			};
			if ( PlusTX )
			{
				if ( mainmode == RXmode )
				{
					if (( param.Modes &  LSBmode ) != CWmode )
					{
						SetIOBW;
					}
					else
					{	
						if (( param.Modes & NARROWmode ) == NARROWmode )
							ResetIOBW;
					};
					if ( ( param.Modes & Compress ) == Compress )
					{
						SetIOKO;
					}
					else
					{
						ResetIOKO;
					};
					AT91F_PIO_SetOutput ( AT91C_BASE_PIOA, aDualLEDred );
					mainmode = TXmode;
					ModeChanged = 1;
					TimeInMainLoop = 0;
				};
			}
			else
			{
				if ( mainmode == TXmode )
				{
					if (( param.Modes & NARROWmode ) == NARROWmode )
						ResetIOBW;
					ResetIOKO;
					AT91F_PIO_ClearOutput ( AT91C_BASE_PIOA, aDualLEDred );
					mainmode = RXmode;
					ModeChanged = 1;
					TimeInMainLoop = 0;
				};
			};

#ifdef QROLLE_COMBO_SPI
			Read_Combo_IO ( B_PORT, (BYTE*)&ComboIOportBIn, 1 );
      if ( ComboIOportBIn & Bug0BitInIO )
        BUG_Conn0_ACTIVE = 0;
      else
        BUG_Conn0_ACTIVE = 1;
      if ( ComboIOportBIn & Bug1BitInIO )
        BUG_Conn1_ACTIVE = 0;
      else
        BUG_Conn1_ACTIVE = 1;
#endif	      
#ifdef QROLLE_COMBO_TWI
      if ( BUG0 )
        BUG_Conn0_ACTIVE = 0;
      else
        BUG_Conn0_ACTIVE = 1;
      if ( BUG1 )
        BUG_Conn1_ACTIVE = 0;
      else
        BUG_Conn1_ACTIVE = 1;
#endif	      
		};
		
		
		
		
    TimeInMainLoop = NUMBERof5msDELAYS;
//   TimeInMainLoop = NUMBERof5msDELAYS*100;
		if ( FIRST_BUTTON ) ButtonActionFlt =  0; else ButtonActionFlt = 1;
		if ( SECOND_BUTTON ) ButtonActionPre =  0; else ButtonActionPre = 1;
		if ( THIRD_BUTTON ) ButtonActionMd =  0; else ButtonActionMd = 1;
		if ( FOURTH_BUTTON ) ButtonActionCom =  0; else ButtonActionCom = 1;
		if ( FIFTH_BUTTON ) ButtonActionABSplit =  0; else ButtonActionABSplit = 1;
		if ( SIXTH_BUTTON ) ButtonActionBdw =  0; else ButtonActionBdw = 1;
		if ( SEVENTH_BUTTON ) ButtonActionstep =  0; else ButtonActionstep = 1;
		if ( EIGHTH_BUTTON ) ButtonActionBup =  0; else ButtonActionBup = 1;
		
		Menu ();  //Check menu buttons, act on changes
		DisplayValues ();
		
		if ( ShowVoltage > 0 )
		{
			ShowVoltageValue ( GetPowerMeterValue ( SMETER_VOLTCHANNEL ), 
												 POS_SMETER );
		}
		else if ( ShowPower )
				ShowSMeterValue ((( param.bPowerSetting * 8 ) / 22 ), GRAPHIC, POS_SMETER );
		else if ( ShowVolume )
				ShowSMeterValue (( Pot1Volume / 22 ), GRAPHIC, POS_SMETER );
		else if (( mainmode == RXmode ) && ( ShowBugSpeed == 0 ))
				ShowSMeterValue ( GetSMeterValue (), SMeterMode, POS_SMETER );
		else if (( mainmode == TXmode ) && ( ShowBugSpeed == 0 ))
		{
			// 164 = 2V/5V * 4096 / 10 ie 2 volt is 10 W, 12 is max displayed
			// 2458 steps equalize 0 to 10 W
			ShowSMeterValue (( 4096 - GetPowerMeterValue (SMETER_ADCCHANNEL)) / 164, 
													POWERMETER, POS_SMETER );
		}
		else if ( ShowBugSpeed > 0 )
		{
			ShowRealBugSpeed ();
		};	
		
		if (( wDisplayBackLiteTime == 1 ) && ( DisplayMode == DisplayAuto ))
		{
			wDisplayBackLiteTime = 0;
			SwitchOFFbacklite;
		};
		SetAD9834value();
		Write2display();
		if ( !timeSinceLastSave )
		{
			timeSinceLastSave = FIVESECONDSIN5MSINCREMENTS;
			if (( AD9834dataDirty ) || 
					( OutputdataDirty ) || 
					( ADCchanneldataDirty ) || 
					( PowerdataDirty ))
			{
				OutputdataDirty = 0;
				ADCchanneldataDirty = 0;
				PowerdataDirty = 0;
				AD9834dataDirty = 0;
				SaveBandData ( Band, AddAD9834Afrequency, sizeof ( struct Parameters ));
			};
			if ( POT1dataDirty )
			{
				POT1dataDirty = 0;
				send2EEPROM ( AddPot1Volume,	(BYTE*)&Pot1Volume, 1 );
			};
		};

   Read_Combo_IO ( A_PORT, (BYTE*)&ComboIOportAIn, 1 );
    if (( ComboIOportAIn & ComboIO_OUTPUT_MASK_A )!= param.IOportA )
	    Write_Combo_IO ( A_PORT, (BYTE*)&param.IOportA, 1 );
		
    Read_Combo_IO ( B_PORT, (BYTE*)&ComboIOportBIn, 1 );
		if (( ComboIOportBIn & ComboIO_OUTPUT_MASK_B )!= param.IOportB )
      Write_Combo_IO ( B_PORT, (BYTE*)&param.IOportB, 1 );
			
		USBHandler (); // Check incoming USB comm
		if ( EndOfCommand_USB ) 
		{
		  if ( CommandHandle_USB () == ERROR_NO_COMMAND )
			{
        TxCommand_USB = 1;
        TransmitByte ( '?' );
        TransmitByte ( ';' );
        TxCommand_USB = 0;
			};																		
		};

		if ( EndOfCommand_RS232 ) 
		{
		  if ( CommandHandle_RS232 () == ERROR_NO_COMMAND )
			{
        TxCommand_RS232 = 1;
        TransmitByte ( '?' );
        TransmitByte ( ';' );
        TxCommand_RS232 = 0;
			};																		
		};
//		while ( PowerdataDirty == 0 );//forever....
	}; // for (;;)
};

//  *******************************************************
//	if  ((AT91C_BASE_PIOA->PIO_ODSR & PA ) == PA )
//	AT91C_BASE_PIOA->PIO_CODR = PA ;							// turn PA  on	
//else
//	AT91C_BASE_PIOA->PIO_SODR = PA ;							// turn PA  off
//  ******************************************************



