/* Copyright (C) 2005 MediTalk AB 

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
//  *******************************************************
//                Header Files
//  *******************************************************

#include "Board.h"
#include <string.h> 
#include "cdc_enumerate.h"
#include "eepromhandler.h"
#include "menu.h"
#include "display.h"
#include "buttons.h"
#include "serialARM.h"


//  *******************************************************
//                Function Prototypes
//  *******************************************************
void Usart0IrqHandler(void);


//  *******************************************************
//                External Globals
//  *******************************************************
extern  char          Buffer[];         // holds received characters
extern  UINT          nChars;           // counts number of received chars
extern  char        * pBuffer;         // pointer into Buffer
volatile BYTE         EndOfCommand_RS232;
volatile BYTE         EndOfCommand_USB;

volatile BYTE         TxCommand_RS232;
volatile BYTE         TxCommand_USB;
volatile BYTE 				 bBaudrate;

//+++++++++++++++++++++++++++++++++++++++++++++++++
//const char cSetBaudrate [] 		=	"Set Baudrate";
//const char cBaudrateNow [] 		=	"Baudrate now ";
//													 		 	 01234567890123456789

#define  NumberOfBaudrates 6
const WORD wBaudrateTable [NumberOfBaudrates] = 
									{ 626, 313, 156, 78, 52, 26 };
const char cBaudrateValue [NumberOfBaudrates][6] = 
									{ "4800  ", "9600  ", "19200 ", "38400 ", "57600 ", "115200" };

/* Static Variables */
static 					BYTE UART_RxBuf [ UART_RX_BUFFER_SIZE ];
static volatile WORD UART_RxHead;
static volatile WORD UART_RxTail;
static 					BYTE UART_TxBuf [ UART_TX_BUFFER_SIZE ];
static volatile WORD UART_TxHead;
static volatile WORD UART_TxTail;
static volatile WORD tmpheadUART;
static volatile WORD tmptailUART;
static volatile BYTE data;

static 					BYTE USB_RxBuf [ USB_RX_BUFFER_SIZE ];
static 					BYTE FromUSB [ USB_RX_BUFFER_SIZE ];
static volatile WORD USB_RxHead;
static volatile WORD USB_RxTail;
static 					BYTE USB_TxBuf [ USB_TX_BUFFER_SIZE ];
static volatile BYTE dataUSB;
static volatile WORD USB_TxCount;
static volatile WORD tmpheadUSB;
static volatile WORD tmptailUSB;

//+++++++++++++++++++++++++++++++++++++++++++++++++


// interrupt handlers 
void Usart0IrqHandler (void) 
{
         
	volatile AT91PS_USART pUsart0 = AT91C_BASE_US0;		// create a pointer to USART0 structure
	
	// determine which interrupt has occurred 
	// assume half-duplex operation here, only one interrupt type at a time
	if ((pUsart0->US_CSR & AT91C_US_RXRDY) == AT91C_US_RXRDY) 
	{
		data = pUsart0->US_RHR; // read the received data 
		// calculate buffer index 
		tmpheadUART = ( UART_RxHead + 1 ) & UART_RX_BUFFER_MASK;
		UART_RxHead = tmpheadUART; // store new index 
		if ( tmpheadUART == UART_RxTail )
		{
			data = pUsart0->US_RHR; // Error buffer overflow 
		};
		
		UART_RxBuf [ tmpheadUART ] = data; // store received data in buffer 
		if ( data == ';' )
		{
		  EndOfCommand_RS232++;
		};
	}; //AT91C_US_RXRDY
	if ((pUsart0->US_CSR & AT91C_US_TXEMPTY) == AT91C_US_TXEMPTY) 
	{
		
		// we have a transmit interrupt (previous char has clocked out)
		// check if all characters have been transmitted
	// check if all data is transmitted 
		if ( UART_TxHead != UART_TxTail )
		{
			// calculate buffer index 
			tmptailUART = ( UART_TxTail + 1 ) & UART_TX_BUFFER_MASK;
			UART_TxTail = tmptailUART; // store new index 
			pUsart0->US_THR = UART_TxBuf [ tmptailUART ]; // start transmition 
		}
		else
		{
			pUsart0->US_IDR = AT91C_US_TXEMPTY;					// disable TX interrupts 
		};
	};
};

//+++++++++++++++++++++++++++++++++++++++++++++++++


// Read and write functions
BYTE ReceiveByte ( void )
{
	WORD tmptail;
	
	// wait for incoming data
	while ( UART_RxHead == UART_RxTail ) ;
	
	tmptail = ( UART_RxTail + 1 ) & UART_RX_BUFFER_MASK;// calculate buffer index
	UART_RxTail = tmptail; // store new index 
	return UART_RxBuf [ tmptail ]; // return data   
};

//+++++++++++++++++++++++++++++++++++++++++++++++++

BYTE DataInReceiveBuffer( void )
{
	return ( UART_RxHead != UART_RxTail ); // return 0 (FALSE) if the receive buffer is empty 
};

//+++++++++++++++++++++++++++++++++++++++++++++++++

void 	InitUSART_ARM ( void )
{
	UART_RxTail = 0;
	UART_RxHead = 0;
	UART_TxTail = 0;
	UART_TxHead = 0;
	EndOfCommand_RS232 = 0;	
	USB_RxTail = 0;
	USB_RxHead = 0;
	USB_TxCount = 0;
	EndOfCommand_USB = 0;	
};
	

//+++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++

void TransmitByte( BYTE data )
{
	WORD tmphead;

	if ( TxCommand_RS232 )
	{
		volatile AT91PS_USART pUsart0 = AT91C_BASE_US0;         // create a pointer to USART0 structure
		
		// calculate buffer index 
		tmphead = ( UART_TxHead + 1 ) & UART_TX_BUFFER_MASK;
	 // wait for free space in buffer
		while ( tmphead == UART_TxTail );
		UART_TxBuf [ tmphead ] = data; // store data in buffer 
		UART_TxHead = tmphead; // store new index 
		
		pUsart0->US_IER = AT91C_US_TXEMPTY;					// enable TXEMPTY usart0 transmit interrupt
	};
	if ( TxCommand_USB )
	{
		if ( USB_TxCount >= USB_TX_BUFFER_SIZE )
		{
			USB_TxCount--;
		};
		USB_TxBuf [ USB_TxCount++ ] = data; // store data in buffer 
		if ( data == ';' )
		{
		  pCDC.Write( &pCDC, 
									&USB_TxBuf [ 0 ] , 
									USB_TxCount );		
			USB_TxCount = 0;
		};
	};
	
};

//+++++++++++++++++++++++++++++++++++++++++++++++++


// Read and write functions
BYTE ReceiveByteUSB ( void )
{
	// wait for incoming data
	while ( USB_RxHead == USB_RxTail ) ;
	
	tmptailUSB = ( USB_RxTail + 1 ) & USB_RX_BUFFER_MASK;// calculate buffer index
	USB_RxTail = tmptailUSB; // store new index 
	return USB_RxBuf [ tmptailUSB ]; // return data  
};

//+++++++++++++++++++++++++++++++++++++++++++++++++

BYTE DataInReceiveBufferUSB( void )
{
	return ( USB_RxHead != USB_RxTail ); // return 0 (FALSE) if the receive buffer is empty 
};

//+++++++++++++++++++++++++++++++++++++++++++++++++

void USBHandler ( void ) 
{
  WORD   BytesReceivedUSB = 0; 
  WORD   BytesInBufferUSB = 0; 
  WORD   incount;
 
  if ( AT91C_BASE_UDP->UDP_GLBSTATE & AT91C_UDP_CONFG )
  {
   if ( AT91F_UDP_Check ( &pCDC ))
  	{
  	  BytesReceivedUSB = AT91F_UDP_Check ( &pCDC );
  		if ( BytesReceivedUSB )
  		{
  			BytesInBufferUSB = pCDC.Read ( &pCDC, &FromUSB [ 0 ], BytesReceivedUSB );
  			for ( incount = 0; incount < BytesInBufferUSB; incount++ )
  			{
  				dataUSB = FromUSB [ incount ]; // read the received data 
  				// calculate buffer index 
  				tmpheadUSB = ( USB_RxHead + 1 ) & USB_RX_BUFFER_MASK;
  				USB_RxHead = tmpheadUSB; // store new index 
  				if ( tmpheadUSB == USB_RxTail )
  				{
  					dataUSB = ' '; // Error buffer overflow 
  				};
  				USB_RxBuf [ tmpheadUSB ] = dataUSB; // store received data in buffer 
  				if ( dataUSB == ';' )
  				{
  					EndOfCommand_USB++;
  				};
  			};//for ( incount = 0; incount < BytesInBufferUSB; incount++ )
  		};//if ( BytesReceivedUSB )	
		};//if ( AT91F_UDP_Check ( &pCDC ))
	};//if ( AT91C_BASE_UDP->UDP_GLBSTATE & AT91C_UDP_CONFG )
};
// *********************************************************************************************************
//              usart0_setup.c
//
//     Purpose:  Set up USART0  (peripheral ID = 6)   9600 baud, 8 data bits, 1 stop bit, no parity
//
//     We will use the onboard baud rate generator to specify 9600 baud
//
//     The Olimex SAM7-EX256 board has a 18,432,000 hz crystal oscillator.
//
//     MAINCK  = 18432000 hz   (from Olimex schematic)
//     DIV = 14                (set up in lowlevelinit.c)
//     MUL = 72                (set up in lowlevelinit.c)
//
//     PLLCK = (MAINCK / DIV) * (MUL + 1) = 18432000/14 * (72 + 1) 
//     PLLCLK = 1316571 * 73 = 96109683 hz
//     MCK = PLLCLK / 2 = 96109683 / 2 = 48054841 hz
//
//     Baud Rate (asynchronous mode) = MCK / (8(2 - OVER)CD)         
//
//      MCK   = 48054841 hz    (set USCLKS = 00 in USART Mode Register US_MR - to select MCK only) 
//      OVER  = 0              (bit 19 of the USART Mode Register US_MR)
//      CD    = divisor        (USART Baud Rate Generator Register US_BRGR)
//          baudrate = 9600        (desired)
//
//                                     48054841            48054841
//    a little algebra:  BaudRate = --------------   =  -----------              
//                                    (8(2 - 0)CD)          16(CD)
//
//                                48054841       48054841
//                         CD  = ----------  =  ---------- = 312.857037               
//                                9600(16)        153600
//
//                         CD = 313  (round up)
//
//                                                  48054841         48054841
//    check the actual baud rate: BaudRate = ---------------  =  ------------ =  9595.6       
//                                              (8 (2 - 0)313          5008
//     
//                                       desired baudrate            9600
//      what's the error:   Error = 1 - ------------------  =  1 - ---------  =  1 - 1.00045854 = -.0004585
//                                       actual baudrate             9595.6                       (not much)
//
//
//
// Author:  James P Lynch  June 22, 2008
// **********************************************************************************************************

void  USART0Setup(void) 
{
  
  // enable the usart0 peripheral clock
  volatile AT91PS_PMC pPMC = AT91C_BASE_PMC;      // pointer to PMC data structure
  pPMC->PMC_PCER = (1<<AT91C_ID_US0);         // enable usart0 peripheral clock
  
  // set up PIO to enable USART0 peripheral control of pins 
  volatile AT91PS_PIO pPIO = AT91C_BASE_PIOA;     // pointer to PIO data structure
  pPIO->PIO_PDR = aRXD0 | aTXD0;  // enable peripheral control of RXD0 and TXD0
  pPIO->PIO_ASR = aRXD0 | aTXD0;    // assigns the 2 I/O lines to peripheral A function
  pPIO->PIO_BSR = 0;                                // peripheral B function set to "no effect"
  
  // set up the USART0 registers
  volatile AT91PS_USART pUSART0 = AT91C_BASE_US0;   // create a pointer to USART0 structure
  
  //    USART0 Control Register US_CR    (read/write)
  //
  //  |-----------------------------------------------------------------------|
  //  |                                                                       |
  //  |-----------------------------------------------------------------------|
  //   31                                                                   24  
  //
  //  |--------|--------|--------|--------|--------|--------|--------|--------|
  //  |                                     RTSDIS   RTSEN    DTRDIS   DTREN  |
  //  |--------|--------|--------|--------|--------|--------|--------|--------|
  //      23       22       21       20       19       18       17       16         
  //
  //  |--------|--------|--------|--------|--------|--------|--------|--------|
  //  | RETTO   RSTNACK   RSTIT    SENDA    STTTO    STPBRK   STTBRK   RSTSTA |
  //  |--------|--------|--------|--------|--------|--------|--------|--------|
  //      15       14       13       12       11       10        9        8         
  //
  //  |--------|--------|--------|--------|--------|--------|--------|--------|
  //  | TXDIS    TXEN     RXDIS    RXEN     RSTTX    RSTRX      -        -    |
  //  |--------|--------|--------|--------|--------|--------|--------|--------|
  //       7          6      5        4        3        2        1        0         
  //
  // RSTRX = 1      (reset receiver)
  // RSTTX = 1      (reset transmitter)
  // RXEN = 0       (receiver enable - no effect)
  // RXDIS = 1      (receiver disable - disabled)
  // TXEN = 0       (transmitter enable - no effect)
  // TXDIS = 1      (transmitter disable - disabled)
  // RSTSTA = 0     (reset status bits - no effect)
  // STTBRK = 0     (start break - no effect)
  // STPBRK = 0     (stop break - no effect)
  // STTTO = 0      (start time-out - no effect)
  // SENDA = 0      (send address - no effect)
  // RSTIT = 0      (reet iterations - no effect)
  // RSTNACK = 0      (reset non acknowledge - no effect)
  // RETTO = 0      (rearm time-out - no effect)
  // DTREN = 0      (data terminal ready enable - no effect)
  // DTRDIS = 0     (data terminal ready disable - no effect)
  // RTSEN = 0      (request to send enable - no effect)
  // RSTDIS = 0     (request to send disable - no effect)

  pUSART0->US_CR =  AT91C_US_RSTRX |    // reset receiver 
                    AT91C_US_RSTTX |    // reset transmitter
                    AT91C_US_RXDIS |    // disable receiver
                    AT91C_US_TXDIS;     // disable transmitter
  
  
  //    USART0 Mode Register US_MR    (read/write)
  //
  //  |--------------------------|--------|--------|--------------------------|
  //  |                            FILTER               MAX_ITERATION         |
  //  |--------------------------|--------|--------|--------------------------|
  //   31                      29    28       27    26                      24      
  //
  //  |--------|--------|--------|--------|--------|--------|--------|--------|
  //  |                   DSNACK    INACK    OVER     CLK0     MODE9    MSBF  |
  //  |--------|--------|--------|--------|--------|--------|--------|--------|
  //      23       22       21       20       19       18       17       16         
  //
  //  |-----------------|-----------------|--------------------------|--------|
  //  |     CHMODE           NBSTOP                 PAR                 SYNC  |
  //  |-----------------|-----------------|--------------------------|--------|
  //   15             14 13             12 11                       9     8         
  //
  //  |-----------------|-----------------|-----------------------------------|
  //  |      CHRL               USCLKS              USART_MODE                |
  //  |-----------------|-----------------|-----------------------------------|
  //    7             6   5             4   3                                0      
  //
  // USART_MODE = 0000  normal
  // USCLKS = 00        choose MCK for baud rate generator
  // CHRL = 11          8-bit characters
  // SYNC = 0           asynchronous mode
  // PAR = 100          no parity
  // NBSTOP = 00        1 stop bit
  // CHMODE = 00        normal mode, no loop-back, etc.
  // MSBF = 0       LSB sent/received first
  // MODE9 = 0          CHRL defines character length
  // CLK0 = 0       USART does not drive SCK pin
  // OVER = 0       16 x oversampling (see baud rate equation)
  // INACK = 0      NACK (not used)
  // DSNACK = 0     not used since NACK is not generated
  // MAX_ITERATION = 0  max iterations not used
  // FILTER = 0     filter is off
  
  pUSART0->US_MR =  AT91C_US_PAR_NONE |   // no parity
                      0x3 << 6;         // 8-bit characters
    
  
  //    USART0 Interrupt Enable Register US_IER    (write only)
  //
  //  |--------|--------|--------|--------|--------|--------|--------|--------|
  //  |                                                                       |
  //  |--------|--------|--------|--------|--------|--------|--------|--------|
  //      31       30       29       28       27       26       25       24         
  //
  //  |--------|--------|--------|--------|--------|--------|--------|--------|
  //  |                                     CTSIC     DCDIC   DSRIC    RIIC   |
  //  |--------|--------|--------|--------|--------|--------|--------|--------|
  //      23       22       21       20       19       18       17       16         
  //
  //  |--------|--------|--------|--------|--------|--------|--------|--------|
  //  |                    NACK    RXBUFF   TXBUFE  ITERATIO TXEMPTY  TIMEOUT |
  //  |--------|--------|--------|--------|--------|--------|--------|--------|
  //      15       14       13       12       11       10        9        8         
  //
  //  |--------|--------|--------|--------|--------|--------|--------|--------|
  //  |  PARE     FRAME    OVRE     ENDTX   ENDRX    RXBRK    TXRDY    RXRDY  |
  //  |--------|--------|--------|--------|--------|--------|--------|--------|
  //       7          6      5        4        3        2        1        0         
  //
  pUSART0->US_IER = 0x00;           // no usart0 interrupts enabled  (no effect) 
  
  
  
  
  //    USART0 Interrupt Disable Register US_IDR    (write only)
  //
  //  |--------|--------|--------|--------|--------|--------|--------|--------|
  //  |                                                                       |
  //  |--------|--------|--------|--------|--------|--------|--------|--------|
  //      31       30       29       28       27       26       25       24         
  //
  //  |--------|--------|--------|--------|--------|--------|--------|--------|
  //  |                                     CTSIC     DCDIC   DSRIC    RIIC   |
  //  |--------|--------|--------|--------|--------|--------|--------|--------|
  //      23       22       21       20       19       18       17       16         
  //
  //  |--------|--------|--------|--------|--------|--------|--------|--------|
  //  |                    NACK    RXBUFF   TXBUFE  ITERATIO TXEMPTY  TIMEOUT |
  //  |--------|--------|--------|--------|--------|--------|--------|--------|
  //      15       14       13       12       11       10        9        8         
  //
  //  |--------|--------|--------|--------|--------|--------|--------|--------|
  //  |  PARE     FRAME    OVRE     ENDTX   ENDRX    RXBRK    TXRDY    RXRDY  |
  //  |--------|--------|--------|--------|--------|--------|--------|--------|
  //       7          6      5        4        3        2        1        0         
  //
  pUSART0->US_IDR = 0xFFFF;         // disable all USART0 interrupts
  
  
  
  //    USART0 Interrupt Mask Register US_IDR    (write only)
  //
  //  |--------|--------|--------|--------|--------|--------|--------|--------|
  //  |                                                                       |
  //  |--------|--------|--------|--------|--------|--------|--------|--------|
  //      31       30       29       28       27       26       25       24         
  //
  //  |--------|--------|--------|--------|--------|--------|--------|--------|
  //  |                                     CTSIC     DCDIC   DSRIC    RIIC   |
  //  |--------|--------|--------|--------|--------|--------|--------|--------|
  //      23       22       21       20       19       18       17       16         
  //
  //  |--------|--------|--------|--------|--------|--------|--------|--------|
  //  |                    NACK    RXBUFF   TXBUFE  ITERATIO TXEMPTY  TIMEOUT |
  //  |--------|--------|--------|--------|--------|--------|--------|--------|
  //      15       14       13       12       11       10        9        8         
  //
  //  |--------|--------|--------|--------|--------|--------|--------|--------|
  //  |  PARE     FRAME    OVRE     ENDTX   ENDRX    RXBRK    TXRDY    RXRDY  |
  //  |--------|--------|--------|--------|--------|--------|--------|--------|
  //       7          6      5        4        3        2        1        0         
  //  read only, nothing to set up here
  
  
  //    USART0 Receive Holding Register  US_RHR    (read only)
  //
  //  |-----------------------------------------------------------------------|
  //  |                                                                       |
  //  |-----------------------------------------------------------------------|
  //   31                                                                   24      
  //
  //  |-----------------------------------------------------------------------|
  //  |                                                                       |
  //  |-----------------------------------------------------------------------|
  //   23                                                                   16      
  //
  //  |--------|-----------------------------------------------------|--------|
  //  | RXSYNH                                                          RXCHR |
  //  |--------|-----------------------------------------------------|--------|
  //      15    14                                                  9     8         
  //
  //  |-----------------------------------------------------------------------|
  //  |                             RXCHR                                  Y  |
  //  |-----------------------------------------------------------------------|
  //   7                                                                     0      
  //  this is where any incoming character will be
  
  
  //    USART0 Transmit Holding Register  US_THR    (write only)
  //
  //  |-----------------------------------------------------------------------|
  //  |                                                                       |
  //  |-----------------------------------------------------------------------|
  //   31                                                                   24      
  //
  //  |-----------------------------------------------------------------------|
  //  |                                                                       |
  //  |-----------------------------------------------------------------------|
  //   23                                                                   16      
  //
  //  |--------|-----------------------------------------------------|--------|
  //  | TXSYNH                                                          TXCHR |
  //  |--------|-----------------------------------------------------|--------|
  //      15    14                                                  9     8         
  //
  //  |-----------------------------------------------------------------------|
  //  |                             TXCHR                                  Y  |
  //  |-----------------------------------------------------------------------|
  //   7                                                                     0      
  //  this is where we place characters to be transmitted

  
  //    USART0 Baud Rate Generator Register US_BRGR    (read/write)
  //
  //  |-----------------------------------------------------------------------|
  //  |                                                                       |
  //  |-----------------------------------------------------------------------|
  //    31                                                                  24      
  //
  //  |--------------------------------------------|--------------------------|
  //  |                                                     FP                |
  //  |--------------------------------------------|--------------------------|
  //   23                                        19 18                      16      
  //
  //  |-----------------------------------------------------------------------|
  //  |                              CD                                       |
  //  |-----------------------------------------------------------------------|
  //   15                                                                    8      
  //
  //  |-----------------------------------------------------------------------|
  //  |                              CD                                    Y  |
  //  |-----------------------------------------------------------------------|
  //   7                                                                     0      

//20090110 pUSART0->US_BRGR = 0x139;   // CD = 0x139  (313 from above calculation)  FP=0 (not used)
 pUSART0->US_BRGR = wBaudrateTable [bBaudrate];


  //    USART0 Receiver Time-out Register US_RTOR    (read/write)
  //
  //  |--------|--------|--------|--------|--------|--------|--------|--------|
  //  |                                                                       |
  //  |--------|--------|--------|--------|--------|--------|--------|--------|
  //      31       30       29       28       27       26       25       24         
  //
  //  |--------|--------|--------|--------|--------|--------|--------|--------|
  //  |                                                                       |
  //  |--------|--------|--------|--------|--------|--------|--------|--------|
  //      23       22       21       20       19       18       17       16         
  //
  //  |-----------------------------------------------------------------------|
  //  |                               TO                                      |
  //  |-----------------------------------------------------------------------|
  //   15                                                                    9      
  //
  //  |-----------------------------------------------------------------------|
  //  |                               TO                                      |
  //  |-----------------------------------------------------------------------|
  //   7                                                                     0      
  //  
  pUSART0->US_RTOR = 0;           // receiver time-out (disabled)
  
  
  //    USART0 transmitter TimeGuard Register US_TTGR    (read/write)
  //
  //  |--------|--------|--------|--------|--------|--------|--------|--------|
  //  |                                                                       |
  //  |--------|--------|--------|--------|--------|--------|--------|--------|
  //      31       30       29       28       27       26       25       24         
  //
  //  |--------|--------|--------|--------|--------|--------|--------|--------|
  //  |                                                                       |
  //  |--------|--------|--------|--------|--------|--------|--------|--------|
  //      23       22       21       20       19       18       17       16         
  //
  //  |--------|--------|--------|--------|--------|--------|--------|--------|
  //  |                                                                       |
  //  |--------|--------|--------|--------|--------|--------|--------|--------|
  //      15       14       13       12       11       10        9       8          
  //
  //  |-----------------------------------------------------------------------|
  //  |                               TG                                      |
  //  |-----------------------------------------------------------------------|
  //   7                                                                     0      
  //  
  pUSART0->US_TTGR = 0;           // transmitter timeguard (disabled)
  
  
  
  //    USART0 FI DI RatioRegister   US_FIDI    (read/write)
  //
  //  |--------|--------|--------|--------|--------|--------|--------|--------|
  //  |                                                                       |
  //  |--------|--------|--------|--------|--------|--------|--------|--------|
  //      31       30       29       28       27       26       25       24         
  //
  //  |--------|--------|--------|--------|--------|--------|--------|--------|
  //  |                                                                       |
  //  |--------|--------|--------|--------|--------|--------|--------|--------|
  //      23       22       21       20       19       18       17       16         
  //
  //  |--------|--------|--------|--------|--------|--------------------------|
  //  |                                                  FI_DI_RATIO          |
  //  |--------|--------|--------|--------|--------|--------------------------|
  //      15       14       13       12       11       10        9       8          
  //
  //  |-----------------------------------------------------------------------|
  //  |                           FI_DI_RATIO                                 |
  //  |-----------------------------------------------------------------------|
  //   7                                                                     0      
  //  not used, nothing to set up here
  
  
  
  //    USART0 Number of Errors Register   US_NER    (read only)
  //
  //  |-----------------------------------------------------------------------|
  //  |                                                                       |
  //  |-----------------------------------------------------------------------|
  //   31                                                                   24      
  //
  //  |-----------------------------------------------------------------------|
  //  |                                                                       |
  //  |-----------------------------------------------------------------------|
  //   23                                                                   16      
  //
  //  |-----------------------------------------------------------------------|
  //  |                                                                       |
  //  |-----------------------------------------------------------------------|
  //   15                                                                    8      
  //
  //  |-----------------------------------------------------------------------|
  //  |                           NB_ERRORS                                   |
  //  |-----------------------------------------------------------------------|
  //   7                                                                     0      
  //  Read-only, nothing to set up here
  
  
  
  //    USART0 IrDA Filter Register  US_IF   (read/write)
  //  |-----------------------------------------------------------------------|
  //  |                                                                       |
  //  |-----------------------------------------------------------------------|
  //   31                                                                   24      
  //
  //  |-----------------------------------------------------------------------|
  //  |                                                                       |
  //  |-----------------------------------------------------------------------|
  //   23                                                                   16      
  //
  //  |-----------------------------------------------------------------------|
  //  |                                                                       |
  //  |-----------------------------------------------------------------------|
  //   15                                                                    8      
  //
  //  |-----------------------------------------------------------------------|
  //  |                           IRDA_FILTER                                 |
  //  |-----------------------------------------------------------------------|
  //   7                                                                     0      
  //  not used, nothing to set up here
  
  // Set up the Advanced Interrupt Controller (AIC)  registers for USART0  
  volatile AT91PS_AIC pAIC = AT91C_BASE_AIC;      // pointer to AIC data structure
  
  pAIC->AIC_IDCR = (1<<AT91C_ID_US0);         // Disable USART0 interrupt in AIC Interrupt Disable Command Register     
  
  pAIC->AIC_SVR[AT91C_ID_US0] =           // Set the USART0 IRQ handler address in AIC Source 
      (unsigned int)Usart0IrqHandler;             // Vector Register[6]
  
  pAIC->AIC_SMR[AT91C_ID_US0] =           // Set the interrupt source type(level-sensitive) and  
      (AT91C_AIC_SRCTYPE_INT_HIGH_LEVEL | 0x4 );    // priority (4)in AIC Source Mode Register[6]
  
  pAIC->AIC_IECR = (1<<AT91C_ID_US0);         // Enable the USART0 interrupt in AIC Interrupt Enable Command Register
  
  // enable the USART0 receiver and transmitter
  pUSART0->US_CR = AT91C_US_RXEN | AT91C_US_TXEN;
  
  // enable the USART0 receive interrupt
  pUSART0->US_IER = AT91C_US_RXRDY;         // enable RXRDY usart0 receive interrupt
  pUSART0->US_IDR = ~AT91C_US_RXRDY;          // disable all interrupts except RXRDY

  InitUSART_ARM ();

  // at this point, only the USART0 receive interrupt is armed!
};


//+++++++++++++++++++++++++++++++++++++++++++++++++

void SetBaudRate ( void )
{
	if ( bBaudrate > (NumberOfBaudrates - 1))
	{
		bBaudrate = 0;
		send2EEPROM ( AddbBaudrate, (BYTE*)&bBaudrate, 1 );
	};
	memset ( (char*)&DisplayBuffer [ 0 ], ' ', CHAR_ON_DISPLAY );
	memcpy ( (char*)&DisplayBuffer [ 0 ], &cSetBaudrate[0], strlen ( cSetBaudrate ));
	memcpy ( (char*)&DisplayBuffer [ CHAR_ON_ONE_LINE ], 
								&cBaudrateNow[0], strlen ( cBaudrateNow ));
	memcpy ( (char*)&DisplayBuffer [ CHAR_ON_ONE_LINE + strlen ( cBaudrateNow )], 
								&cBaudrateValue[bBaudrate][0], 6 );

		if ( menuPosCol == 0 ) 
			return;

	if ( CheckEncoderBYTE ( (BYTE*)&bBaudrate, NumberOfBaudrates - 1, 1 )) 
	{
	};
	if ( KnobGoingActive )
	{
		memset ( (char*)&DisplayBuffer [ 0 ], ' ', CHAR_ON_DISPLAY );
		KnobGoingActive = false;
		AlreadyFound = true;
		AT91C_BASE_US0->US_BRGR = wBaudrateTable [bBaudrate];
		send2EEPROM ( AddbBaudrate, (BYTE*)&bBaudrate, 1 );
		if ( menuPosRow > 1 )
			menuPosRow--;
		menuPosCol = 0;
	};
};
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	

